# -*- coding: utf-8 -*-
import sys
sys.path.insert(0, 'controller/')
sys.path.insert(0, 'persistence/')
import time
import telepot
import telepot.helper
from telepot.loop import MessageLoop
from telepot.namedtuple import ForceReply
from telepot.delegate import (per_chat_id, per_callback_query_origin, per_inline_from_id, create_open, pave_event_space)

from model.checklist import Checklist, States
from model.chat import Chat
import controller, chatsController, checklistsController, tasksController, assignTaskController, viewChecklistsController
import checklistsManager, chatsManager

Chat.Chats = chatsManager.findAll()

'''
Clase MsgHandler. Trata todas las peticiones del usuario de tipo mensaje (texto, imagen, sticker, etc.)
'''
class MsgHandler(telepot.helper.ChatHandler):
    '''
    Constructor de la clase. Hereda de ChatHandler
    '''
    def __init__(self, *args, **kwargs):
        super(MsgHandler, self).__init__(*args, **kwargs)
        
    '''
    Recibe y contesta al mensaje proporcionado
    @param message: mensaje recibido del usuario
    ''' 
    def on_chat_message(self, message):
        content_type, chat_type, _id = telepot.glance(message)
        from_id = _id
        
        #Comprobacion mensaje de grupo (chat y miembro)
        if message['chat']['type'] == 'group':
            from_id = message['from']['id']
            msg = chatsController.messageGroup(bot, _id, from_id)
            if msg != None:
                if msg['markup'] == None:
                    self.sender.sendMessage(msg['text'])
                else:
                    self.sender.sendMessage(msg['text'], reply_markup = msg['markup'])
        else:
            #Comprobacion nuevo chat (conversacion) con bot  
            msg = chatsController.newChat(bot, _id)
            if msg != None:
                self.sender.sendMessage(msg['text'], reply_markup = msg['markup'])
        
        if content_type == 'left_chat_member':
            chatsController.messageLeftMember(_id, message['left_chat_member']['id'])
        
        elif content_type == 'text':
            if message['chat']['type'] != 'group' or (message['chat']['type'] == 'group' and (message['text'][0] == '/' or 'reply_to_message' in message)):
                args = message['text'].split()
                
                if args[0] == '/new':
                    msg = checklistsController.newChecklist(_id, args)
                    if msg['params'] == None:
                        self.sender.sendMessage(msg['text'])
                    elif msg['markup'] == None:
                        self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'])
                    else:
                        self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                        
                elif args[0] == '/assigntask': 
                    msg = assignTaskController.selectChecklist(_id, from_id, message['chat']['type'])
                    if msg['markup'] == None: 
                        self.sender.sendMessage(msg['text'])
                    else:
                        self.sender.sendMessage(msg['text'], reply_markup= msg['markup'])
                 
                elif args[0] == '/checklists':
                    msg = viewChecklistsController._all(_id, from_id)
                    self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'])
                    
                elif args[0] == '/help':
                    msg = chatsController._help(_id)
                    self.sender.sendMessage(msg['text'])
                    
                elif args[0] == '/language':
                    msg = chatsController.selectLanguage(_id)
                    self.sender.sendMessage(msg['text'], reply_markup = msg['markup'])
                   
                elif args[0][0:5] == '/view':
                    argsList = args[0].split('_')
                    cList = checklistsManager.findById(argsList[1])
                    msg = viewChecklistsController.detail(_id, cList)
                    if msg != None:
                        if msg['markup'] == None:
                            self.sender.sendMessage(parse_mode = 'HTML', text = msg['text']) 
                        else:
                            msg = self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                            msg_id = telepot.message_identifier(msg)
                            cList.replaceMsg(msg_id)
                            checklistsManager.update(cList)
                            Chat.Chats[_id].setIdList(None)
                            
                elif Chat.Chats[_id].getIdList() == '':
                    msg = checklistsController.createChecklist(_id, message['text'])
                    if msg['params'] == None:
                        self.sender.sendMessage(msg['text'])
                    elif msg['markup'] == None:
                        self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'])
                    else:
                        self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                
                elif Chat.Chats[_id].getIdList() != None:   
                    idList = Chat.Chats[_id].getIdList()
                    nameTask = Chat.Chats[_id].getNameTask()
                    try:
                        cList = Checklist.Checklists[idList]
                    except KeyError:
                        cList = checklistsManager.findById(idList)
    
                    if 'reply_to_message' in message:
                        msg = assignTaskController.assign(_id, from_id, nameTask, cList, message)
                        iMsg = Chat.Chats[_id].getInlineMsg()
    
                        for i in range(len(iMsg)):
                            msg_id = telepot.message_identifier(iMsg[i])
                            try:
                                self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg[i]['text'] % msg[i]['params'])
                            except telepot.exception.TelegramError:
                                self.sender.sendMessage(parse_mode='HTML', text = msg[i]['text'] % msg[i]['params'])
                            
                    elif cList.getState() == States.CREATE.name and args[0] == '/done': #Checklist terminada
                        msg = checklistsController.doneChecklist(_id, cList)
                        msg = self.sender.sendMessage(parse_mode = 'HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                        msg_id = telepot.message_identifier(msg)
                        cList.replaceMsg(msg_id)
                        cList.setState(4)
                        checklistsManager.update(cList)
                        Chat.Chats[_id].setIdList(None)
                   
                    elif cList.getState() == States.CREATE.name or cList.getState() == States.ADDTASK.name: #Añade tarea
                        msg = tasksController.newTask(_id, message['text'], cList)
                        if msg!= None:
                            if not 'text' in msg:
                                messages = msg
                                for msg_id in messages:
                                    msg = messages[msg_id]
                                    if msg_id[0] == _id:
                                        msg = self.sender.sendMessage(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                                    else:
                                        try:
                                            msg = self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                                        except telepot.exception.TelegramError:
                                            cList.removeMsg(msg_id)
                                            
                                    if not isinstance(msg, bool):
                                        msg_id = telepot.message_identifier(msg)
                                        cList.replaceMsg(msg_id)
                                
                                cList.setState(4)
                                checklistsManager.update(cList)
                                Chat.Chats[_id].setIdList(None)
                                        
                            elif msg['params'] == None:
                                self.sender.sendMessage(msg['text'])
                            else:
                                self.sender.sendMessage(parse_mode='HTML', text = msg['text'] % msg['params'])
                    
                    elif cList.getState() == States.EDIT.name and nameTask != None: #Edita texto tarea
                        msg = tasksController.modify(_id, nameTask, message['text'], cList.getId())
                        if msg != None:
                            if not 'text' in msg:
                                messages = msg
                                for msg_id in messages:
                                    msg = messages[msg_id]
                                    if msg_id[0] == _id:
                                        iMsg = Chat.Chats[_id].getInlineMsg()
                                        iMsg_id = telepot.message_identifier(iMsg)
                                        self.bot.editMessageText(iMsg_id, parse_mode='HTML', text = iMsg['text'])
                                        msg = self.sender.sendMessage(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                                    else:
                                        try:
                                            msg = self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                                        except telepot.exception.TelegramError:
                                            cList.removeMsg(msg_id)
                                            
                                    if not isinstance(msg, bool):
                                        msg_id = telepot.message_identifier(msg)
                                        cList.replaceMsg(msg_id)
                                        
                                cList.setState(4)
                                checklistsManager.update(cList)
                                Chat.Chats[_id].setIdList(None)
                                
                            else:
                                self.sender.sendMessage(parse_mode='HTML', text = msg['text'] % msg['params'])
                    
                else:
                    msg = controller.notUnderstand(_id)
                    self.sender.sendMessage(msg['text'])

'''
Clase CallbackHandler. Trata todas las peticiones del usuario de tipo callback_query
'''
class CallbackHandler(telepot.helper.CallbackQueryOriginHandler):
    '''
    Constructor de la clase. Hereda de CallbackQueryOriginHandler
    '''
    def __init__(self, *args, **kwargs):
        super(CallbackHandler, self).__init__(*args, **kwargs)
     
    '''
    Recibe y resuelve la peticion callback_query proporcionada
    @param message: mensaje con la peticion callback_query recibida
    '''
    def on_callback_query(self, message):
        query_id, _id, data = telepot.glance(message, flavor='callback_query')
        args = data.split('-')
        chat_id = _id
        
        try:
            if message['message']['chat']['type'] == 'group':
                chat_id = message['message']['chat']['id']
        except KeyError:
            chat_id = 0
        
        chatsController.newChat(bot, _id)
            
        if args[0] == 'NewChecklist':
            msg = checklistsController.createChecklist(_id, args[1])
            self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'])
                
        elif args[0] == 'Language':
            msg = chatsController.saveLanguage(chat_id, args[1])
            self.editor.editMessageText(msg['text'] % msg['params'])
                
        else:
            cList = checklistsManager.findById(args[1])
            
            if args[0] == 'Edit':
                msg = checklistsController.editOptions(_id, cList)
                if msg['markup'] == None:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'])
                else:
                    self.editor.editMessageText(msg['text'], reply_markup = msg['markup'])
                self.bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'AddTask':
                msg = tasksController.requestNewTask(_id, cList)
                self.editor.editMessageText(msg['text'])
                self.bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'ModifyTask':
                msg = tasksController.selectTask(_id, cList.getId())
                self.editor.editMessageText(msg['text'], reply_markup = msg['markup'])
                self.bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'Modify':
                msg = tasksController.requestModifyTask(_id, args[2], cList)
                iMsg = self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                self.bot.answerCallbackQuery(query_id)
                Chat.Chats[chat_id].setInlineMsg(iMsg)
                
            elif args[0] == 'UndoTask':
                messages = tasksController.undo(_id, args[1], args[2])
                for msg_id in messages:
                    msg = messages[msg_id]
                    try:
                        self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                    except telepot.exception.TelegramError:
                        cList.removeMsg(msg_id)
                        
                self.bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'RemoveTask_Unconfirmed':
                msg = tasksController.selectRemoveTask(_id, cList)
                if msg['params'] == None:
                    self.editor.editMessageText(msg['text'], reply_markup = msg['markup'])
                else:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'RemoveTask':
                messages = tasksController.remove(_id, args[2], cList.getId())
                for msg_id in messages:
                    msg = messages[msg_id]
                    try:
                        self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                    except telepot.exception.TelegramError:
                        cList.removeMsg(msg_id)
                 
                cList.setState(4)
                checklistsManager.update(cList)
                Chat.Chats[_id].setIdList(None)
                bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'ClearList':
                messages = checklistsController.clear(_id, cList.getId())
                for msg_id in messages:
                    msg = messages[msg_id]
                    try:
                        self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                    except telepot.exception.TelegramError:
                        cList.removeMsg(msg_id)
                        
                self.bot.answerCallbackQuery(query_id)
               
            elif args[0] == 'RemoveList_Unconfirmed':
                msg = checklistsController.requestRemoveListConfirmation(_id, cList)
                self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                self.bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'RemoveList':
                msg = checklistsController.remove(_id, cList)
                self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'])
                self.bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'No':
                msg = chatsController.noModify(_id, cList)
                self.editor.editMessageText(parse_mode='HTML', text = '%s' % msg['params'], reply_markup = msg['markup'])
                self.bot.answerCallbackQuery(query_id, msg['text'] % cList.getName(), show_alert=False)
            
            elif args[0] == 'Back':
                msg = controller.checklistMessage(chat_id, int(args[1]), '%s', False)
                if msg != None:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup=msg['markup'])
                bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'Clone':
                msg = checklistsController.cloneChecklist(_id, cList)
                if msg != None:
                    if msg['params'] == None:
                        self.editor.editMessageText(msg['text'])
                    else:
                        self.bot.answerCallbackQuery(query_id, msg['text'] % msg['params'], show_alert=True)
                
            elif args[0] == 'AssignToList':
                msg = assignTaskController.selectTask(chat_id, _id, cList)
                if msg['markup'] == None:
                    self.bot.answerCallbackQuery(query_id, msg['text'], show_alert=True)
                else:
                    self.editor.editMessageText(msg['text'], reply_markup = msg['markup'])
                    self.bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'AssignTask':
                msg = assignTaskController.selectMember(bot, chat_id, _id, args[2], cList)
                if msg['markup'] == None:
                    self.bot.answerCallbackQuery(query_id, msg['text'], show_alert=True)
                else:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                    self.bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'Assign':
                msg = assignTaskController.assign_CallbackQuery(chat_id, _id, args[2], cList, args[3])
                if msg['params'] == None:
                    self.bot.answerCallbackQuery(query_id, msg['text'], show_alert=True)
                else:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'])
                    self.bot.answerCallbackQuery(query_id)
            
            elif args[0] == 'AddMember':
                msg = assignTaskController.addMember(chat_id, _id, cList)
                if 'params' in msg:
                    self.bot.answerCallbackQuery(query_id, msg['text'], show_alert=True)
                else:
                    iMsg = [self.editor.editMessageText(msg[0]['text'])]
                    iMsg.append(bot.sendMessage(chat_id, parse_mode='HTML', text = msg[1]['text'], reply_markup = ForceReply()))
                    self.bot.answerCallbackQuery(query_id)
                    Chat.Chats[chat_id].setInlineMsg(iMsg)
                
            elif args[0] == 'Check':
                msg = tasksController.check(_id, chat_id, args[2], cList)
                if msg != None:
                    if not 'text' in msg:
                        messages = msg
                        for msg_id in messages:
                            msg = messages[msg_id]
                            try:
                                msg = self.bot.editMessageText(msg_id, parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup = msg['markup'])
                            except telepot.exception.TelegramError:
                                cList.removeMsg(msg_id)
                            
                            if not isinstance(msg, bool):
                                msg_id = telepot.message_identifier(msg)
                                cList.replaceMsg(msg_id)
                            
                        checklistsManager.update(cList)
                        Chat.Chats[_id].setIdList(None)
                        self.bot.answerCallbackQuery(query_id, 'La tarea %s fue realizada' % args[2], show_alert=False)
                        
                    elif msg['params'] == None and msg['markup'] == None:
                        self.bot.answerCallbackQuery(query_id, msg['text'], show_alert=True)
                    elif msg['markup'] == None:
                        self.bot.answerCallbackQuery(query_id, msg['text'] %  msg['params'], show_alert=True)
                    else:
                        self.editor.editMessageText(msg['text'])
                        
            elif args[0] == 'Previous':
                #Al hacer split por '-' si el numero es negativo args[2] = '' -> no actualizamos el msg
                if args[2] != '':
                    msg = controller.checklistMessage(chat_id, int(args[1]), '%s', False, page = int(args[2]))
                    if msg != None:
                        self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup=msg['markup'])
                self.bot.answerCallbackQuery(query_id)
                
            elif args[0] == 'Next':
                msg = controller.checklistMessage(chat_id, int(args[1]), '%s', False, page = int(args[2]))
                if msg != None:
                    self.editor.editMessageText(parse_mode='HTML', text = msg['text'] % msg['params'], reply_markup=msg['markup'])
                self.bot.answerCallbackQuery(query_id)

'''
Clase InlineHandler. Trata todas las peticiones del usuario de tipo inline
'''
class InlineHandler(telepot.helper.InlineUserHandler, telepot.helper.AnswererMixin):
    '''
    Constructor de la clase. Hereda de InlineUserHandler
    '''
    def __init__(self, *args, **kwargs):
        super(InlineHandler, self).__init__(*args, **kwargs)

    '''
    Recibe y contesta a la peticion inline proporcionada
    @param message: mensaje con la peticion inline recibida
    '''
    def on_inline_query(self, message):
        
        def compute_answer():
            query_id, _id, query = telepot.glance(message, flavor='inline_query')
            return viewChecklistsController.findResults(_id, query)
            
        self.answerer.answer(message, compute_answer)

    '''
    Recibe la seleccion de un usuario a una peticion inline
    @param message: mensaje con el resultado seleccionado de los proporcionados por la peticion inline
    '''
    def on_chosen_inline_result(self, message):
        result_id, _id, query_string = telepot.glance(message, flavor='chosen_inline_result')
        viewChecklistsController.share(message['inline_message_id'], result_id)
        
TOKEN = '350435553:AAFeISB-ZAk6CwsRo3goXA7vz2mOpHqrd_s'

bot = telepot.DelegatorBot(TOKEN, [
    pave_event_space()(
        per_chat_id(), create_open, MsgHandler, timeout=10),
    pave_event_space()(
        per_callback_query_origin(), create_open, CallbackHandler, timeout=60),
    pave_event_space()(
        per_inline_from_id(), create_open, InlineHandler, timeout=10),
])

MessageLoop(bot).run_as_thread()

# Keep the program running.
while True:
	time.sleep(10)