import telepot
from telepot.namedtuple import InlineKeyboardMarkup, InlineKeyboardButton

from model.task import Task
from model.checklist import Checklist
from model.chat import Chat
from model.user import MyUser
import resources
import controller
import chatsManager, usersManager, tasksManager, checklistsManager

'''
Retorna un mensaje acompañado de una lista de botones con todas las checklists del usuario indicado
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario del cual hay que recuperar la checklists
@param chatType: string con el tipo de chat
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
'''
def selectChecklist(_id, from_id, chatType):
    if chatType == 'group':
        auxKeyboard = []
        for cList in checklistsManager.findAllByOwner(from_id):
            auxKeyboard.append([InlineKeyboardButton(text=str(cList.getName()), callback_data= 'AssignToList-' + str(cList.getId()))])
                
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        return controller.message(resources.getResource('AssignChecklist', Chat.Chats[_id].getLanguage()), None, markup)
    else:
        return controller.message(resources.getResource('Error_PrivateChat', Chat.Chats[_id].getLanguage()), None, None)

'''
Retorna un mensaje acompañado de una lista de botones con todas las tareas pertenecientes a la lista indicada
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que ha lanzado la callback_query
@param cList: objeto checklist del que hay que recuperar sus tareas
@return message: diccionario con los valores necesarios para responder a la petición del usuario
'''        
def selectTask(_id, from_id, cList):
    if from_id == cList.getOwner():
        auxKeyboard = []
        for task in tasksManager.findAllByList(cList.getId()):
            auxKeyboard.append([InlineKeyboardButton(text=str(task.getName()), 
                callback_data= 'AssignTask-' + str(cList.getId()) + '-' + str(task.getName()))])
                    
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        return controller.message(resources.getResource('WhatTask3', Chat.Chats[_id].getLanguage()), None, markup)
    else:
        return controller.message(resources.getResource('Error_NoOwner', Chat.Chats[_id].getLanguage()), None, None)

'''
Retorna un mensaje acompañado de una lista de botones con todos los usuarios miembros del grupo
@param bot: objeto DelegatorBot que tiene todos los métodos de la Api Bot Telegram
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que ha lanzado la callback_query
@param nameTask: nombre de la tarea que se va a asignar
@param cList: objeto checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
'''    
def selectMember(bot, _id, from_id, nameTask, cList):
    if from_id == cList.getOwner():
        auxKeyboard = []
        for member in Chat.Chats[_id].getMembers():
            user = usersManager.findById(member)
            if user != None:
                nameUser = user.getName()
            
            auxKeyboard.append([InlineKeyboardButton(text= nameUser, 
                callback_data= 'Assign-' + str(cList.getId()) + '-' + str(nameTask) + '-' + str(member))])
        
        auxKeyboard.append([InlineKeyboardButton(text= resources.getResource('Nobody', Chat.Chats[_id].getLanguage()), 
            callback_data= 'Assign-' + str(cList.getId()) + '-' + str(nameTask) + '-None')])
    
        text = resources.getResource('WhomAssign', Chat.Chats[_id].getLanguage())
    
        #Se resta 1 para no contabilizar al propio bot en el grupo, informa de si el bot tiene
        #almacenados datos de todos los miembros del grupo
        if len(Chat.Chats[_id].getMembers()) < bot.getChatMembersCount(_id) - 1:
            text += resources.getResource('Group_Incomplete', Chat.Chats[_id].getLanguage())
            auxKeyboard.append([InlineKeyboardButton(text = resources.getResource('AddMember', Chat.Chats[_id].getLanguage()),
                callback_data = 'AddMember-' + str(cList.getId()))])
        
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        Chat.Chats[_id].setIdList(cList.getId())
        Chat.Chats[_id].setNameTask(nameTask)
        return controller.message(text, (nameTask, cList.getName()), markup)
    else:
        return controller.message(resources.getResource('Error_NoOwner', Chat.Chats[_id].getLanguage()), None, None)

'''
Retorna un mensaje informando de si el usuario ha podido ser asignado a la tarea o no 
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que ha enviado el mensaje
@param nameTask: nombre de la tarea que se va a asignar
@param cList: objeto checklist al cual pertenece la tarea
@param message: mensaje que ha enviado el usuario
@return result: diccionario con los valores necesarios para responder al mensaje del usuario
'''
def assign(_id, from_id, nameTask, cList, message):
    if from_id == cList.getOwner():
        task = tasksManager.findByNameAndIdList(nameTask, cList.getId())
        if task != None:
            try:
                entity = message['entities'][0]['user']
            except (KeyError, IndexError):
                #Usuario con alias, error de asignacion
                return [controller.message(resources.getResource('Error_MemberNotAdded', 
                        Chat.Chats[_id].getLanguage()), message['text'], None),
                        controller.message(resources.getResource('Error_AssignTaskUsername', 
                        Chat.Chats[_id].getLanguage()), message['text'], None)]
    
            owner = entity['id']
            nameOwner = entity['first_name']
            task.setOwner(owner)
            tasksManager.updateTask(task, None)
            
            #Añade usuario a miembros del grupo si no estaba
            if not Chat.Chats[_id].isMember(owner):    
                Chat.Chats[_id].joinGroup(owner)
                chatsManager.updateMembersChat(Chat.Chats[_id])
                
            #Inserta elemento usuario con datos de este si no estaba
            if usersManager.findById(owner) == None:
                try:
                    username = entity['username']
                except KeyError:
                    username = None
                    
                usersManager.insert(MyUser(owner, nameOwner, username))
                
            Chat.Chats[_id].setIdList(None)
            result = [controller.message(resources.getResource('MemberAdded', 
                        Chat.Chats[_id].getLanguage()), nameOwner, None),
                    controller.message(resources.getResource('Task_Assigned', 
                        Chat.Chats[_id].getLanguage()), (nameTask, nameOwner), None)]
            return result
    else:
        return controller.message(resources.getResource('Error_NoOwner', Chat.Chats[_id].getLanguage()), None, None)
    
'''
Retorna un mensaje informando del usuario que ha sido asignado a la tarea
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que ha enviado el mensaje
@param nameTask: nombre de la tarea que se va a asignar
@param cList: objeto checklist al cual pertenece la tarea
@param member: miembro del grupo al cual se va a asignar la tarea
@return result: diccionario con los valores necesarios para responder a la peticion del usuario
'''   
def assign_CallbackQuery(_id, from_id, nameTask, cList, member):
    if from_id == cList.getOwner():
        task = tasksManager.findByNameAndIdList(nameTask, cList.getId())
        if task != None:
            if member != 'None':
                owner = member
            else:
                owner = None
                
            task.setOwner(owner)
            tasksManager.updateTask(task, None)
            if member != 'None':
                user = usersManager.findById(owner)
                if user != None:
                    nameOwner = user.getName()
                else:
                    nameOwner = owner
                    
                return controller.message(resources.getResource('Task_Assigned',Chat.Chats[_id].getLanguage()),(nameTask, nameOwner), None)
            else:
                return controller.message(resources.getResource('Task_Unassigned',Chat.Chats[_id].getLanguage()), nameTask, None)
    else:
        return controller.message(resources.getResource('Error_NoOwner', Chat.Chats[_id].getLanguage()), None, None)
 
'''
Retorna un mensaje solicitando el usuario al cual se va a asignar la tarea 
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que ha enviado el mensaje
@param cList: objeto checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder a la peticion del usuario
'''       
def addMember(_id, from_id, cList):
    if from_id == cList.getOwner():
        result = [controller.message(resources.getResource('AddingMember1',Chat.Chats[_id].getLanguage()), None, None),
                controller.message(resources.getResource('AddingMember2',Chat.Chats[_id].getLanguage()), None, None)]
        return result
    else:
        return controller.message(resources.getResource('Error_NoOwner', Chat.Chats[_id].getLanguage()), None, None)