import telepot
from telepot.namedtuple import InlineKeyboardMarkup, InlineKeyboardButton

from model.chat import Chat
from model.user import MyUser
import resources
import controller
import chatsManager, usersManager

'''
Comprueba si se tienen datos almacenados de un usuario, si no es así, 
inserta los datos que Telegram devuelve sobre este
@param bot: objeto DelegatorBot que tiene todos los métodos de la Api Bot Telegram
@param _id: identificador del usuario
'''
def checkUser(bot, _id):
    if usersManager.findById(_id) == None:
        user = bot.getChatMember(_id, _id)
        try:
            username = user['user']['username']
        except KeyError:
            username = None
        usersManager.insert(MyUser(user['user']['id'], user['user']['first_name'], username))

'''
Retorna un mensaje acompañado de una lista de botones con los lenguajes disponibles
@param _id: identificador del chat con el que se está interactuando
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
'''
def selectLanguage(_id):
    auxKeyboard = [[InlineKeyboardButton(text='English', callback_data= 'Language-EN')],
                        [InlineKeyboardButton(text='Español', callback_data= 'Language-ES')]]
    markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    
    return controller.message(resources.getResource('SelectLanguage', Chat.Chats[_id].getLanguage()), None, markup)

'''
Retorna un mensaje informando del lenguaje seleccionado. Almacena el lenguaje relacionado al chat.
@param _id: identificador del chat con el que se está interactuando
@param language: string con el lenguaje seleccionado
@return message: diccionario con los valores necesarios para responder a la peticion del usuario
'''
def saveLanguage(_id, language):
    Chat.Chats[_id].setLanguage(language)
    chatsManager.updateLanguageChat(Chat.Chats[_id])
    return controller.message(resources.getResource('Language_Selected', 
        Chat.Chats[_id].getLanguage()), language, None)

'''
Retorna un mensaje informando de que la lista no ha sido modificada.
@param _id: identificador del chat con el que se está interactuando
@param cList: checklist que mantiene su estado antes de comenzar las modificaciones
@return message: diccionario con los valores necesarios para responder a la peticion del usuario
'''       
def noModify(_id, cList):
    markup = controller.checklistKeyboard(_id, cList, 0, True)
    return controller.message(resources.getResource('Checklist_NotEdited', Chat.Chats[_id].getLanguage()), controller.printChecklist(cList), markup)

'''
Comprueba si se tienen datos almacenados de un chat, si no es así, inserta un nuevo elemento chat
y retorna un mensaje con el resultado de llamar al método selectLanguage().
@param bot: objeto DelegatorBot que tiene todos los métodos de la Api Bot Telegram
@param _id: identificador del chat con el que se está interactuando
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
'''    
def newChat(bot, _id):
    if _id not in Chat.Chats:
        Chat.Chats[_id] = Chat(_id)
        chatsManager.insert(Chat.Chats[_id])
        #Los id de grupo son negativos, solo se quiere almacenar en el XML
        #users los datos de usuario NO grupos.
        if _id > 0:
            checkUser(bot, _id)
        return selectLanguage(_id)

'''
Retorna un mensaje con la ayuda del bot.
@param _id: identificador del chat con el que se está interactuando
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
'''   
def _help(_id):
    return controller.message(resources.getResource('Help', Chat.Chats[_id].getLanguage()), None, None)

'''
Comprueba si usuario es miembro de un grupo, si no es así, lo inserta como miembro.
@param _id: identificador del chat con el que se está interactuando
@param from_id: identificador del usuario que se va a comprobar
@return message: diccionario con los valores necesarios para responder al mensaje del usuario
''' 
def messageGroup(bot, _id, from_id):
    #id del checklistbot
    if from_id != 350435553:
        if _id in Chat.Chats:
            if not Chat.Chats[_id].isMember(from_id):
                Chat.Chats[_id].joinGroup(from_id)
                chatsManager.updateMembersChat(Chat.Chats[_id])
                checkUser(bot, from_id)
                return controller.message(resources.getResource('NewMember', Chat.Chats[_id].getLanguage()), None, None)
        else:
            Chat.Chats[_id] = Chat(_id)
            Chat.Chats[_id].joinGroup(from_id)
            chatsManager.insert(Chat.Chats[_id])
            checkUser(bot, from_id)
            return selectLanguage(_id)

'''
Elimina un miembro si este estaba almacenado.
@param _id: identificador del chat con el que se está interactuando
@param idMember: identificador del usuario que se va a comprobar
'''    
def messageLeftMember(_id, idMember):
    if Chat.Chats[_id].isMember(idMember):
        Chat.Chats[_id].leftGroup()
        chatsManager.updateMembersChat(Chat.Chats[_id])