import random
from telepot.namedtuple import InlineKeyboardMarkup, InlineKeyboardButton

from model.task import Task
from model.checklist import Checklist
from model.chat import Chat
import resources
import controller
import checklistsManager, tasksManager

'''
Genera tres nuevos nombres, que no tenga ya ninguna otra checklist del usuario indicado, 
a raiz del nombre proporcionado
@param name: nombre ya existente para la checklist 
@param owner: identificador del usuario
@return validNames: array con 3 nombres de checklist 
'''
def generateName(name, owner):
    NUMBERS = '1234567890_'
    validNames = []
    for i in range(3):
        auxName = name
        while True:
            if checklistsManager.findByNameAndOwner(auxName, str(owner)) != None:
                auxName += random.choice(NUMBERS)
                
            elif auxName not in validNames:
                    validNames.append(auxName)
                    break
                
    return validNames

'''
Busca el primer identificador de checklist disponible
'''
def getFirstFreeId():
    checklists = checklistsManager.findAll()
    if len(checklists) > 0:
        i = checklists[-1].getId()
        if i >= Checklist.idList:
            if checklistsManager.findById(i + 1) == None and i not in Checklist.Checklists:
                Checklist.idList = i + 1

'''
Realiza las comprobaciones necesarias para crear una nueva checklist. Retorna un
mensaje informando de si la lista se ha creado o no y por qué
@param _id: identificador del usuario/chat con el que se está interactuando 
@param name: nombre para la nueva checklist
@return message: diccionario con los valores necesarios para responder al mensaje del usuario 
'''        
def createChecklist(_id, name):
    if len(name) > 50:
            return controller.message(resources.getConcatResources(['Error_ChecklistLength', 'Name_Required'], 
                Chat.Chats[_id].getLanguage()), None, None)
    
    elif checklistsManager.findByNameAndOwner(name, _id) == None:
        getFirstFreeId()
        cList = Checklist(name, _id, None)
        Checklist.Checklists[cList.getId()] = cList
        Chat.Chats[_id].setIdList(cList.getId())
        
        return controller.message(resources.getConcatResources(['Checklist_Created', 'FirstTask'], 
            Chat.Chats[_id].getLanguage()), name, None)
    
    else:
        auxKeyboard = []
        for validName in generateName(name, _id):
            auxKeyboard.append([InlineKeyboardButton(text=str(validName), callback_data= 'NewChecklist-' + str(validName))])
        
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        return controller.message(resources.getResource('Name_Duplicate', Chat.Chats[_id].getLanguage()), name, markup)
   
'''
Retorna un mensaje solicitando el nombre de la checklist si esque no se ha proporcionado junto al comando.
Si se ha proporcionado el nombre, se llama a createChecklist() para validarlo
@param _id: identificador del usuario/chat con el que se está interactuando 
@param args: mensaje enviado por el usuario
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def newChecklist(_id, args):
    if _id < 0:
        return controller.message(resources.getResource('Error_GroupChat', Chat.Chats[_id].getLanguage()), None, None)
    if len(args)==1:
        Chat.Chats[_id].setIdList('')
        return controller.message(resources.getResource('Name_Required', Chat.Chats[_id].getLanguage()), None, None)
    else:
        name = ' '.join(args[1 - len(args):]) 
        return createChecklist(_id, name)

'''
Retorna un mensaje acompañado de una lista de botones con todas las opciones de edición disponibles
@param _id: identificador del usuario/chat con el que se está interactuando 
@param cList: objeto checklist que se va a editar
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def editOptions(_id, cList):
    if cList != None:
        auxKeyboard = [[InlineKeyboardButton(text= resources.getResource('AddTask', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'AddTask-' + str(cList.getId()))],
                        [InlineKeyboardButton(text= resources.getResource('ModifyTask', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'ModifyTask-' + str(cList.getId()))],
                        [InlineKeyboardButton(text= resources.getResource('RemoveTask', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'RemoveTask_Unconfirmed-' + str(cList.getId()))],
                        [InlineKeyboardButton(text= resources.getResource('ClearList', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'ClearList-' + str(cList.getId()))],
                        [InlineKeyboardButton(text= resources.getResource('RemoveList', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'RemoveList_Unconfirmed-' + str(cList.getId()))],
                        [InlineKeyboardButton(text= resources.getResource('OnBack', Chat.Chats[_id].getLanguage()), 
                            callback_data= 'Back-' + str(cList.getId()))]]
        
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        Chat.Chats[_id].setIdList(cList.getId())
        return controller.message(resources.getResource('How_Edit', Chat.Chats[_id].getLanguage()), None, markup)
    else:
        return controller.message(resources.getResource('Checklist_NotFound', Chat.Chats[_id].getLanguage()), None, None)

'''
Reinicializa la checklist y retorna un mensaje con la checklist actualizada
@param _id: identificador del usuario/chat con el que se está interactuando 
@param idList: identificador de checklist que se va a reinicializar
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def clear(_id, idList):
    for task in tasksManager.findAllByList(idList):
        task.setChecked(False)
        tasksManager.updateTaskState(task)
      
    return controller.checklistMessage(_id, idList, resources.getResource('Checklist_Cleared', 
        Chat.Chats[_id].getLanguage()), True)

'''
Retorna un mensaje acompañado de una lista de botones solicitando la confirmacion de eliminacion de una checklist
@param _id: identificador del usuario/chat con el que se está interactuando 
@param cList: objeto checklist que se quiere eliminar
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def requestRemoveListConfirmation(_id, cList):
    auxKeyboard = [[InlineKeyboardButton(text=resources.getResource('Yes', Chat.Chats[_id].getLanguage()), 
                        callback_data= 'RemoveList'+ '-' + str(cList.getId()))],
                   [InlineKeyboardButton(text=resources.getResource('No', Chat.Chats[_id].getLanguage()), 
                        callback_data= 'No'+ '-' + str(cList.getId()))]]                    
    markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    return controller.message(resources.getResource('RemoveList_Confirmation', Chat.Chats[_id].getLanguage()), cList.getName(), markup)

'''
Elimina la checklist y retorna un mensaje informando de la acción
@param _id: identificador del usuario/chat con el que se está interactuando 
@param cList: objeto checklist que se va a eliminar
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def remove(_id, cList):
    #Simula un deleteOnCascade
    for task in tasksManager.findAllByList(cList.getId()):
        tasksManager.delete(task.getName(), cList.getId())
        
    checklistsManager.delete(cList.getId())
    Task.Tasks.pop(cList.getId(), None)
    Checklist.Checklists.pop(cList.getId(), None)
    return controller.message(resources.getResource('Checklist_Removed', Chat.Chats[_id].getLanguage()), cList.getName(), None)

'''
Confirma la creación de la checklist y retorna un mensaje con la nueva checklist
@param _id: identificador del usuario/chat con el que se está interactuando 
@param cList: objeto checklist que se ha creado
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def doneChecklist(_id, cList):
    checklistsManager.insert(cList)
    
    #Simula un createOnCascade
    for task in Task.Tasks[cList.getId()]:
        tasksManager.insert(task)
     
    #Borra objetos de memoria una vez cargados en XML   
    Task.Tasks.pop(cList.getId(), None)
    Checklist.Checklists.pop(cList.getId(), None)
    return controller.checklistMessage(_id, cList.getId(), resources.getResource('Checklist_Finished', 
        Chat.Chats[_id].getLanguage()), False)

'''
Clona una checklist, si se puede, y retorna un mensaje informando del resultado de la acción
@param _id: identificador del usuario/chat con el que se está interactuando 
@param cList: objeto checklist que se quiere clonar
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
'''         
def cloneChecklist(_id, cList):
    if cList != None:
        if checklistsManager.findByNameAndOwner(cList.getName(), _id) == None:
            getFirstFreeId()
            cListClon = Checklist(cList.getName(), _id, None)
            
            for task in tasksManager.findAllByList(cList.getId()):
                taskClon = Task(task.getName(), cListClon.getId())
                tasksManager.insert(taskClon)
                
            checklistsManager.insert(cListClon)
            return controller.message(resources.getResource('Checklist_Cloned', Chat.Chats[_id].getLanguage()), cList.getName(), None)
        
        else:
            return controller.message(resources.getResource('Checklist_ClonedDuplicate', Chat.Chats[_id].getLanguage()), cList.getName(), None)
    
    else:
        return controller.message(resources.getResource('Error_ListNotFound', Chat.Chats[_id].getLanguage()), None, None)
