from telepot.namedtuple import InlineKeyboardMarkup, InlineKeyboardButton

from model.task import Task
from model.checklist import Checklist, States
from model.chat import Chat
import resources
import controller
import usersManager, tasksManager, checklistsManager

'''
Crea una tarea y retorna un mensaje informando del resultado de la acción
@param _id: identificador del usuario/chat con el que se está interactuando 
@param nameTask: nombre de la nueva tarea
@param cList: objeto checklist al cual pertenece la nueva tarea
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def newTask(_id, nameTask, cList):
    #Comprueba si la tarea existe ya en la lista
    inList = False
    if cList.getId() in Task.Tasks:
        tasks = Task.Tasks[cList.getId()]
        for task in tasks:
            if task.getName() == nameTask:
                inList = True
                break
    else:
        if tasksManager.findByNameAndIdList(nameTask, cList.getId()) != None:
            inList = True
            
    if inList == False:
        if len(nameTask) > 50:
            return controller.message(resources.getConcatResources(['Error_TaskLength','Checklist_AddTask'], 
                Chat.Chats[_id].getLanguage()), None, None)
                
        else:
            task = Task(nameTask, cList.getId())
            
            if cList.getState() == States.ADDTASK.name: #Añade tarea a una lista ya creada
                tasksManager.insert(task)
                return controller.checklistMessage(_id, cList.getId(), resources.getCompleteResource('Checklist_NewTask', [nameTask],
                Chat.Chats[_id].getLanguage()), True)
            
            elif cList.getState() == States.CREATE.name: #Añade tarea a una lista en creacion 
                try:
                    Task.Tasks[cList.getId()].append(task)
                except KeyError:
                    Task.Tasks[cList.getId()] = [task]
                
                return controller.message(resources.getConcatResources(['Checklist_AddTask','Checklist_Done'], 
                    Chat.Chats[_id].getLanguage()), None, None)
    
    else:
        if cList.getState() == States.ADDTASK.name: #ERROR:Tarea duplicada en una lista ya creada
            return controller.message(resources.getConcatResources(['Task_Duplicate','Checklist_AddTask'], 
                Chat.Chats[_id].getLanguage()), nameTask, None)
            
        elif cList.getState() == States.CREATE.name: #ERROR:Tarea duplicada en una lista en creacion 
            return controller.message(resources.getConcatResources(['Task_Duplicate','Checklist_AddTask', 'Checklist_Done'], 
                Chat.Chats[_id].getLanguage()), nameTask, None)

'''
Retorna un mensaje solicitando la nueva tarea que se quiere añadir
@param _id: identificador del usuario/chat con el que se está interactuando
@param cList: objeto checklist al cual pertenecerá la nueva tarea
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def requestNewTask(_id, cList):
    Chat.Chats[_id].setIdList(cList.getId())
    cList.setState(2) #ADDTASK
    checklistsManager.updateState(cList.getId(), cList.getState())
    return controller.message(resources.getResource('AddingTask', Chat.Chats[_id].getLanguage()), None, None)

'''
Retorna un mensaje acompañado de una lista de botones con las tareas pertenecientes a la checklist proporcionada
@param _id: identificador del usuario/chat con el que se está interactuando
@param idList: identificador de checklist del cual se buscan sus tareas
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def selectTask(_id, idList):
    auxKeyboard = []
    for task in tasksManager.findAllByList(idList):
        auxKeyboard.append([InlineKeyboardButton(text=str(task.getName()), callback_data= 'Modify-'+ str(idList) + '-' + str(task.getName()))])
    
    markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    return controller.message(resources.getResource('WhatTask1', Chat.Chats[_id].getLanguage()), None, markup)

'''
Retorna un mensaje solicitando la nueva tarea que reemplazará a la indicada
@param _id: identificador del usuario/chat con el que se está interactuando
@param nameTask: nombre de la tarea que se va a reemplazar
@param cList: objeto checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def requestModifyTask(_id, nameTask, cList):
    Chat.Chats[_id].setIdList(cList.getId())
    Chat.Chats[_id].setNameTask(nameTask)
    cList.setState(3) #EDIT
    checklistsManager.updateState(cList.getId(), cList.getState())
    auxKeyboard = [[InlineKeyboardButton(text=resources.getResource('UndoTask', Chat.Chats[_id].getLanguage()), callback_data= 'UndoTask-'+ str(cList.getId()) + '-' + nameTask)]]
    markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    return controller.message(resources.getResource('ModifyingTask', Chat.Chats[_id].getLanguage()), nameTask, markup)
 
'''
Deshace la tarea indicada y retorna un mensaje con la checklist actualizada
@param _id: identificador del usuario/chat con el que se está interactuando
@param idList: identificador de checklist al cual pertenece la tarea
@param nameTask: nombre de la tarea que se deshace
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
'''    
def undo(_id, idList, nameTask):
    task = tasksManager.findByNameAndIdList(nameTask, idList)
    task.setChecked(False)
    tasksManager.updateTaskState(task)
    return controller.checklistMessage(_id, idList, resources.getCompleteResource('Checklist_UndoTask', [nameTask],
            Chat.Chats[_id].getLanguage()), True)

'''
Reemplaza el nombre de la tarea indicada por el nuevo y retorna un mensaje con la checklist actualizada
@param _id: identificador del chat con el que se está interactuando 
@param nameTask: nombre de la tarea a modificar
@param newName: nombre nuevo para la tarea
@param idList: identificador de checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder a la peticion del usuario 
'''               
def modify(_id, nameTask, newName, idList):
    if tasksManager.findByNameAndIdList(newName, idList) == None or nameTask == newName:
        inList = False
    else:
        inList = True
            
    if inList == False:
        task = tasksManager.findByNameAndIdList(nameTask,idList)
        tasksManager.updateTask(task, newName)
        return controller.checklistMessage(_id, idList, resources.getCompleteResource('Checklist_EditTask', [nameTask],
            Chat.Chats[_id].getLanguage()), True)
    else:
        return controller.message(resources.getConcatResources(['Task_Duplicate','Checklist_AddTask'], 
                Chat.Chats[_id].getLanguage()), newName, None)

'''
Retorna un mensaje acompañado de una lista de botones con las tareas de la checklist indicada
para que el usuario seleccione cual borrar
@param _id: identificador del chat con el que se está interactuando 
@param cList: objeto checklist del cual se buscan las tareas
@return message: diccionario con los valores necesarios para responder a la peticion del usuario 
'''    
def selectRemoveTask(_id, cList):
    tasks = tasksManager.findAllByList(cList.getId())
    if len(tasks) > 1:
        auxKeyboard = []
        for task in tasks:
            auxKeyboard.append([InlineKeyboardButton(text=str(task.getName()), callback_data= 'RemoveTask-'+ str(cList.getId()) + '-' + str(task.getName()))])
    
        markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
        return controller.message(resources.getResource('WhatTask2', Chat.Chats[_id].getLanguage()), None, markup)
        
    else:
        return requestRemoveTaskConfirmation(_id, cList, tasks[0].getName())

'''
Retorna un mensaje solicitando confirmacion para eliminar una tarea cuando esta es la última
@param _id: identificador del chat con el que se está interactuando 
@param cList: objeto checklist del cual se buscan las tareas
@param nameTask: nombre de la tarea a eliminar
@return message: diccionario con los valores necesarios para responder a la peticion del usuario 
'''  
def requestRemoveTaskConfirmation(_id, cList, nameTask):
    auxKeyboard = [[InlineKeyboardButton(text=resources.getResource('Yes', Chat.Chats[_id].getLanguage()), 
                        callback_data= 'RemoveList'+ '-' + str(cList.getId()))],
                   [InlineKeyboardButton(text=resources.getResource('No', Chat.Chats[_id].getLanguage()), 
                        callback_data= 'No'+ '-' + str(cList.getId()))]]
                        
    markup = InlineKeyboardMarkup(inline_keyboard=auxKeyboard)
    return controller.message(resources.getResource('LastTask', Chat.Chats[_id].getLanguage()), (nameTask, cList.getName()), markup)

'''
Elimina la tarea indicada de la lista y retorna un mensaje con la checklist actualizada
@param _id: identificador del chat con el que se está interactuando 
@param nameTask: nombre de la tarea a eliminar
@param idList: identificador de checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder a la peticion del usuario 
'''     
def remove(_id, nameTask, idList):
    tasksManager.delete(nameTask, idList)
    return controller.checklistMessage(_id, idList, resources.getCompleteResource('Checklist_DeleteTask', [nameTask], 
        Chat.Chats[_id].getLanguage()), True)

'''
Marca el estado de la tarea indicada a "done" y retorna un mensaje con la checklist actualizada
@param _id: identificador del usuario que ha marcado la tarea
@chat_id: identificador del chat con el que se esta interactuando
@param nameTask: nombre de la tarea marcada
@param cList: objeto checklist al cual pertenece la tarea
@return message: diccionario con los valores necesarios para responder a la peticion del usuario 
'''     
def check(_id, chat_id, nameTask, cList):
    if cList != None:
        task = tasksManager.findByNameAndIdList(nameTask, cList.getId())
        if task != None:
            owner = task.getOwner()
            if not task.isChecked():  
                if str(owner) in (str(None), str(_id)):
                    task.setChecked(True)
                    tasksManager.updateTaskState(task)
                    return controller.checklistMessage(chat_id, cList.getId(), '%s', True)
                else:
                    user = usersManager.findById(owner)
                    if user != None:
                        name = user.getName()
                    else:
                        name = owner
                    return controller.message(resources.getResource('Error_CheckAssigned', Chat.Chats[_id].getLanguage()), name, None)
            else:
                return controller.message(resources.getResource('Error_TaskChecked', Chat.Chats[_id].getLanguage()), None, None)
        
    else:
        return controller.message(resources.getResource('Error_ListNotFound', Chat.Chats[_id].getLanguage()), None, 0)