from model.task import Task
from model.checklist import Checklist
from model.chat import Chat
from model.user import MyUser
import resources
import controller
import checklistsManager, tasksManager

'''
Calcula el numero de tareas con estado "done" de todas las tareas proporcionadas
@param tasks: array de objetos Task  
@return done: entero con el numero de tareas realizadas 
''' 
def doneTasks(tasks):
    done = 0
    for t in tasks:
        if t.isChecked():
            done += 1
    return done

'''
Retorna un mensaje con todas las checklists del usuario indicado, informacion sobre cada una 
y un comando para recuperar cada checklist
@param _id: identificador del chat con el que se está interactuando  
@param from_id: identificador del usuario del cual se buscan sus listas
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def _all(_id, from_id):
    checklists = checklistsManager.findAllByOwner(from_id)
    if _id < 0:
        msg = resources.getResource('Error_GroupChat', Chat.Chats[_id].getLanguage())
    elif len(checklists) == 0:
        msg = resources.getResource('Error_AnyLists', Chat.Chats[_id].getLanguage())
    else:
        msg =  resources.getResource('Checklists', Chat.Chats[_id].getLanguage())
        count = 1
        for cList in checklists:
            url = '/view_' +  str(cList.getId())
            tasks = tasksManager.findAllByList(cList.getId())
            if len(tasks) == 0:
                checklistsManager.delete(cList.getId())
            else:   
                msg +=  str(count) + '.' + '<b>' + str(cList.getName()) + '</b> ' + str(doneTasks(tasks)) + '/' + str(len(tasks)) + resources.getResource('Tasks_Done', Chat.Chats[_id].getLanguage()) + url + '\n\n'
                count += 1
    return controller.message(msg, None, None)

'''
Retorna un mensaje con la checklist indicada o un mensaje de error si el _id no se corresponde con el del creador
@param _id: identificador del chat/usuario con el que se está interactuando  
@param cList: objeto checklist del cual se devuelven sus datos
@return message: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def detail(_id, cList):
    if cList != None:
        if _id == cList.getOwner():
            return controller.checklistMessage(_id, cList.getId(), '%s', False)
        else:
            return controller.message(resources.getResource('Error_ListNotFoundView', Chat.Chats[_id].getLanguage()), None, None)     
    else: 
        return controller.message(resources.getResource('Error_ListNotFoundView', Chat.Chats[_id].getLanguage()), None, None)  

'''
Retorna un diccionario con las checklists obtenidas de la busqueda
@param _id: identificador del chat/usuario con el que se está interactuando  
@param query: string con la busqueda realizada por el usuario
@return diccionario: diccionario con los valores necesarios para responder a la petición del usuario 
''' 
def findResults(_id, query):
    results = []
    checklists = checklistsManager.findAllByOwner(_id)
    
    for cList in checklists:
        letters = len(query)
        if cList.getName()[:letters].upper() == query.upper():
            r = {'type': 'article',
                'id': str(cList.getId()), 
                'title': cList.getName(), 
                'message_text': controller.printChecklist(cList),
                'parse_mode': 'HTML',
                'reply_markup': controller.checklistKeyboard(_id, cList, 0)}
            results.append(r)
        
        if len(results) == 5:
            break
        
    return {'results':results, 'cache_time':0}

'''
Añade el id del mensaje a los mensajes de una checklist para poder realizar las actualizaciones futuras
@param msg_idf: identificador del mensaje con la checklist  
@param idList: identificador de checklist a la que se va a añadir el mensaje 
''' 
def share(msg_idf, idList):
    cList = checklistsManager.findById(idList)
    cList.appendMsg(msg_idf)
    checklistsManager.update(cList)